/*
 *  linux/arch/arm/mach-integrator/leds.c
 *
 *  Integrator LED control routines
 *
 *  Copyright (C) 1999 ARM Limited
 *  Copyright (C) 2000 Deep Blue Solutions Ltd
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/kernel.h>
#include <linux/init.h>

#include <asm/hardware.h>
#include <asm/leds.h>
#include <asm/system.h>
#include <asm/mach-types.h>

#include <asm/arch/bits.h>
#include <asm/arch/sizes.h>
#include <asm/arch/platform.h>

static int led_state;
static int saved_leds;

#define LEDS_BASE (IO_ADDRESS(INTEGRATOR_DBG_BASE)+INTEGRATOR_DBG_LEDS_OFFSET)
#define ALPHA_BASE (IO_ADDRESS(INTEGRATOR_DBG_BASE)+INTEGRATOR_DBG_ALPHA_OFFSET)

#if 0 /* is this used? */
void set_leds(unsigned int value)
{
        volatile unsigned int *leds = (volatile unsigned int *)LEDS_BASE;

	// update the bits
	saved_leds = value;

	// get the old led settings
	*leds = saved_leds;
}

unsigned int get_leds(void)
{
        volatile unsigned int *leds = (volatile unsigned int *)LEDS_BASE;

	return *leds;
}
#endif

void integrator_leds_event(led_event_t ledevt)
{
	unsigned long flags;
	volatile unsigned int *alpha = (volatile unsigned int *)ALPHA_BASE;
        volatile unsigned int *leds = (volatile unsigned int *)LEDS_BASE;
	
	// yup, change the LEDs
	local_irq_save(flags);
	switch(ledevt) {
	case led_idle_start:
		saved_leds |= YELLOW_LED;
		break;

	case led_idle_end:
		saved_leds &= ~YELLOW_LED;
		break;

	case led_timer:
		saved_leds ^= GREEN_LED;
		break;
	default:
		break;
	}

	local_irq_restore(flags);

	while (*alpha & 1);

	// update the LEDs
	*leds = saved_leds;
}

static int __init leds_init(void)
{
	if (machine_is_integrator())
		leds_event = integrator_leds_event;

	return 0;
}

__initcall(leds_init);
